Package: shrinkTVP
Type: Package
Title: Efficient Bayesian Inference for Time-Varying Parameter Models
        with Shrinkage
Version: 3.1.2
Authors@R: c(
  person("Peter", "Knaus", email = "peter.knaus@wu.ac.at",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6498-7084")),
  person("Angela", "Bitto-Nemling", role = "aut"),
  person("Annalisa", "Cadonna", email = "annalisa.cadonna@wu.ac.at",
    role = "aut", comment = c(ORCID = "0000-0003-0360-7628")),
  person("Sylvia", "Frühwirth-Schnatter", email = "sylvia.fruehwirth-schnatter@wu.ac.at",
    role = "aut", comment = c(ORCID = "0000-0003-0516-5552")),
  person("Daniel", "Winkler", email = "daniel.winkler@wu.ac.at", role = "ctb"),
  person("Kemal", "Dingic", role = "ctb"))
Description: Efficient Markov chain Monte Carlo (MCMC) algorithms for fully Bayesian estimation of time-varying parameter models with shrinkage priors, both dynamic and static. Details on the algorithms used are provided in Bitto and Frühwirth-Schnatter (2019) <doi:10.1016/j.jeconom.2018.11.006> and  
  Cadonna et al. (2020) <doi:10.3390/econometrics8020020> and Knaus and Frühwirth-Schnatter (2023)  <doi:10.48550/arXiv.2312.10487>. For details on the package, please see Knaus et al. (2021) <doi:10.18637/jss.v100.i13>. For the multivariate extension, see the 'shrinkTVPVAR' package.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.3.0)
Imports: Rcpp, GIGrvg, stochvol (>= 3.0.3), coda, methods, utils, zoo
LinkingTo: Rcpp, RcppArmadillo, GIGrvg, RcppProgress, stochvol, RcppGSL
RoxygenNote: 7.3.2
Suggests: testthat, knitr, rmarkdown, R.rsp
VignetteBuilder: R.rsp
NeedsCompilation: yes
Packaged: 2026-07-13 14:10:53 UTC; knaus
Author: Peter Knaus [aut, cre] (ORCID: <https://orcid.org/0000-0001-6498-7084>),
  Angela Bitto-Nemling [aut],
  Annalisa Cadonna [aut] (ORCID: <https://orcid.org/0000-0003-0360-7628>),
  Sylvia Frühwirth-Schnatter [aut] (ORCID:
    <https://orcid.org/0000-0003-0516-5552>),
  Daniel Winkler [ctb],
  Kemal Dingic [ctb]
Maintainer: Peter Knaus <peter.knaus@wu.ac.at>
Repository: CRAN
Date/Publication: 2026-07-13 16:00:02 UTC
