# Generated by roxygen2: do not edit by hand

S3method(fitted,shrinkTVP)
S3method(plot,mcmc.tvp)
S3method(plot,shrinkTVP)
S3method(plot,shrinkTVP_forc)
S3method(predict,shrinkTVP)
S3method(print,shrinkTVP)
S3method(print,summary.shrinkTVP)
S3method(residuals,shrinkTVP)
S3method(summary,shrinkTVP)
export(LPDS)
export(eval_pred_dens)
export(forecast_shrinkTVP)
export(shrinkDTVP)
export(shrinkTVP)
export(simTVP)
export(updateTVP)
import(coda)
importFrom(GIGrvg,rgig)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,adjustcolor)
importFrom(graphics,abline)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(methods,formalArgs)
importFrom(stats,.getXlevels)
importFrom(stats,delete.response)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.pass)
importFrom(stats,quantile)
importFrom(stats,rf)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,sd)
importFrom(stochvol,svsample)
importFrom(utils,maintainer)
importFrom(zoo,index)
useDynLib(shrinkTVP, .registration = TRUE)
