#' @useDynLib shrinkTVP, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#'
#' @importFrom stochvol svsample
#'
#' @importFrom GIGrvg rgig
#'
#' @import coda
#'
#' @importFrom graphics plot lines abline par polygon layout
#'
#' @importFrom grDevices adjustcolor
#'
#' @importFrom stats quantile model.response model.matrix model.frame sd median rnorm na.pass delete.response .getXlevels rf rgamma rpois
#'
#' @importFrom methods formalArgs
#'
#' @importFrom utils maintainer
#'
#' @importFrom zoo index
NULL
